const express = require("express");
const Game = require("./game.js");

class TicTacToe extends Game {
  constructor(id) {
    super(id);
    this.teams = [
      {
        id: 0,
        name: "X"
      },
      {
        id: 1,
        name: "O"
      }];
    this.state = {
      moves: [],
      board: Array(9).fill(""),
      team_to_play: this.teams[0],
    };
  }

  /* Returns true if move was legal and added, false otherwise. */
  add_move(player, square) {

    const state = this.state;
    const result = super.add_move(player, square);

    /* If the generic Game class can reject this move, then we don't
     * need to look at it any further. */
    if (! result.legal)
      return result;

    /* Cannot move to an occupied square. */
    if (state.board[square])
    {
      return { legal: false,
               message: "Square is already occupied" };
    }

    state.board[square] = state.team_to_play;
    state.moves.push(square);

    if (state.team_to_play.id === 0)
      state.team_to_play = this.teams[1];
    else
      state.team_to_play = this.teams[0];

    return { legal: true };
  }
}

TicTacToe.router = express.Router();

TicTacToe.meta = {
  name: "Tic Tac Toe",
  identifier: "tictactoe",
  options: {
    allow_guest: true
  }
};

exports.Game = TicTacToe;
